'=========
'10x10.bas - v1.8
'=========
'A 10x10 lines puzzle (Woody clone)
'Coded by Dav, FEB/2022

'v1.8 changes     * Game progress saved and can be resumed later.
'                 * New menu system.  Right click to call menu.
'                   Or click on the 'Menu' at the top right.
'                   Pressing M will also bring up the menu.
'                   All settings now done by clicking menu system.
'                 * Current sound setting saved to setting file.
'                 * 10x10.dat contains game scores & sfx setting.
'                 * After game is over, the menu will pop up.
'                 * Added How to Play section in the menu.
'                 * Added some messages when bonus points scored.
'                   (like the original woody game does it)
'                 * All needed files now come external, not in .BAS.
'
'                 This may be the final version - the game is looking
'                 and playing how I envisioned it.  So unless there's
'                 an error to correct I'd say this game is done.

'=====
'ABOUT:
'=====

'This is a clone of the Woody puzzle.  Drag pices onto the board
'to make completed rows or columns.  Completed lines will erase.
'Bonus points given for clearing more than one line at a time.
'Game ends when there's no space left of the board for pieces.
'See how high a score you can get before running out of space.
'
'Game progress is saved, so you can quit and resume later.
'
'Click on the 'Menu' to right up the menu of options.
'You may also right click the mouse, or press M to bring menu.


RANDOMIZE TIMER

SCREEN _NEWIMAGE(800, 650, 32)

_DELAY 1

CLS , _RGB(193, 94, 47)

_TITLE "10x10"

DIM SHARED rows, cols, size, piece1, piece2, piece3, score, hiscore, sfx

rows = 10: cols = 10: size = 60

DIM SHARED box(rows * cols)
DIM SHARED box.x1(rows * cols), box.x2(rows * cols)
DIM SHARED box.y1(rows * cols), box.y2(rows * cols)
DIM SHARED litesquare&, backboard&, pieceup&, piecedown&, menu&, howto&
DIM SHARED menuitem1on&, menuitem1off&, menuitem2on&, menuitem2off&
DIM SHARED menuitem3on&, menuitem3off&, menuitem4on&, menuitem4off&
DIM SHARED menuitem5on&, menuitem5off&, tophi&, topscore&, topmenu&
DIM SHARED outofmoves1&, outofmoves2&

litesquare& = _LOADIMAGE("10x10-piece.png", 32)
backboard& = _LOADIMAGE("10x10-board.png", 32)
menu& = _LOADIMAGE("10x10-menu2.png", 32)
howto& = _LOADIMAGE("10x10-howtoplay.png", 32)
pieceup& = _SNDOPEN("10x10-pieceup.wav")
piecedown& = _SNDOPEN("10x10-piecedown.wav")
menuitem1on& = _LOADIMAGE("10x10-item1-on.png", 32)
menuitem1off& = _LOADIMAGE("10x10-item1-off.png", 32)
menuitem2on& = _LOADIMAGE("10x10-item2-on.png", 32)
menuitem2off& = _LOADIMAGE("10x10-item2-off.png", 32)
menuitem3on& = _LOADIMAGE("10x10-item3-on.png", 32)
menuitem3off& = _LOADIMAGE("10x10-item3-off.png", 32)
menuitem4on& = _LOADIMAGE("10x10-item4-on.png", 32)
menuitem4off& = _LOADIMAGE("10x10-item4-off.png", 32)
menuitem5on& = _LOADIMAGE("10x10-item5-on.png", 32)
menuitem5off& = _LOADIMAGE("10x10-item5-off.png", 32)
topscore& = _LOADIMAGE("10x10-top-score.png", 32)
tophi& = _LOADIMAGE("10x10-top-hi.png", 32)
topmenu& = _LOADIMAGE("10x10-top-menu.png", 32)
outofmoves1& = _LOADIMAGE("10x10-outofmoves1.png", 32)
outofmoves2& = _LOADIMAGE("10x10-outofmoves2.png", 32)

firststart = 1

'============
Restart:
'============

'Init the box values
bc = 1 'counter
FOR c = 1 TO cols
    FOR r = 1 TO rows
        x = (r * size): y = 50 + (c * size)
        box.x1(bc) = x - size: box.x2(bc) = x
        box.y1(bc) = y - size: box.y2(bc) = y
        box(bc) = 0 'set value to 0...means empty box
        bc = bc + 1
    NEXT
NEXT

'fail safes
hiscore = 0
score = 0
sfx = 1

'=== check if.dat file is currupt
'    if not big enough, erasde it
IF _FILEEXISTS("10x10.dat") THEN
    scr = FREEFILE
    OPEN "10x10.dat" FOR INPUT AS #scr
    IF LOF(scr) <> (12 + (rows * cols)) THEN KILL "10x10.dat"
    CLOSE scr
END IF

'=== load scores and settings from file
IF _FILEEXISTS("10x10.dat") THEN
    scr = FREEFILE
    OPEN "10x10.dat" FOR BINARY AS #scr
    IF LOF(scr) = (12 + (rows * cols)) THEN
        hiscore = CVL(INPUT$(4, scr))
        score = CVL(INPUT$(4, scr))
        sfx = CVL(INPUT$(4, scr))
        'get saved box values
        bc = 1 'counter
        FOR c = 1 TO cols
            FOR r = 1 TO rows
                box(bc) = VAL(INPUT$(1, scr))
                bc = bc + 1
            NEXT
        NEXT
    END IF
    CLOSE #scr
END IF

ShowScore
DrawHoldingPens
UpdateBoard

DO

    IF firststart = 1 THEN
        firststart = 0
        MainMenu
        IF score = 0 THEN GOTO Restart
    END IF

    t = _MOUSEINPUT

    IF _MOUSEBUTTON(1) THEN
        mx = _MOUSEX: my = _MOUSEY

        'if clicked holding pen 1
        IF mx >= 610 AND mx <= 790 AND my >= 60 AND my <= 240 THEN

            IF piece1 <> 0 THEN

                'grab background in case restore needed
                undo& = _COPYIMAGE(_DISPLAY)

                'clear holding pen 1
                LINE (610, 60)-(790, 240), _RGB(105, 14, 0), BF: _DISPLAY

                back& = _COPYIMAGE(_DISPLAY)

                'pickup piece, drag it around until mouse up

                IF sfx = 1 THEN _SNDPLAY pieceup&

                DO
                    'show piece in hand, over backgraound
                    IF INT(RND * 3) = 0 THEN
                        _PUTIMAGE (0, 0), back&
                        '_PUTIMAGE (_MOUSEX, _MOUSEY), litesquare&: _DISPLAY
                        DrawPiece piece1, _MOUSEX, _MOUSEY
                    END IF

                    'If user unclicked the button (let go)
                    t2 = _MOUSEINPUT
                    IF _MOUSEBUTTON(1) = 0 THEN
                        mx2 = _MOUSEX: my2 = _MOUSEY

                        'see if unclicked over an empty box
                        FOR r = 1 TO rows * cols
                            IF mx2 >= box.x1(r) AND mx2 <= box.x2(r) AND my2 >= box.y1(r) AND my2 <= box.y2(r) THEN

                                'If the piece fits...
                                IF Validate(piece1, r, 0) = 1 THEN

                                    IF sfx = 1 THEN _SNDPLAY piecedown&

                                    _PUTIMAGE (0, 0), back&: _DISPLAY
                                    AddToScore piece1
                                    piece1 = 0 'remove piece from pen
                                ELSE
                                    curpiece = piece1
                                    GOSUB PieceReturn
                                    _PUTIMAGE (0, 0), undo&: _DISPLAY
                                END IF

                                GOTO doneamove

                            END IF
                        NEXT
                        curpiece = piece1
                        GOSUB PieceReturn

                        _PUTIMAGE (0, 0), undo&: _DISPLAY

                        GOTO doneamove

                    END IF

                LOOP
                'please release me, let me go....
                _FREEIMAGE undo&
                _FREEIMAGE back&
            END IF
        END IF

        'if clicked holding pen 2
        IF mx >= 610 AND mx <= 790 AND my >= 260 AND my <= 440 THEN

            IF piece2 <> 0 THEN

                'grab background in case restore needed
                undo& = _COPYIMAGE(_DISPLAY)

                'clear holding pen 2
                LINE (610, 260)-(790, 440), _RGB(105, 14, 0), BF: _DISPLAY

                back& = _COPYIMAGE(_DISPLAY)

                IF sfx = 1 THEN _SNDPLAY pieceup&

                'pickup piece, drag it around until mouse up
                DO
                    'show piece in hand, over backgraound
                    IF INT(RND * 3) = 0 THEN
                        _PUTIMAGE (0, 0), back&
                        '_PUTIMAGE (_MOUSEX, _MOUSEY), litesquare&: _DISPLAY
                        DrawPiece piece2, _MOUSEX, _MOUSEY
                    END IF

                    'If user unclicked the button (let go)
                    t2 = _MOUSEINPUT
                    IF _MOUSEBUTTON(1) = 0 THEN
                        mx2 = _MOUSEX: my2 = _MOUSEY

                        'see if unclicked over an empty box
                        FOR r = 1 TO rows * cols
                            IF mx2 >= box.x1(r) AND mx2 <= box.x2(r) AND my2 >= box.y1(r) AND my2 <= box.y2(r) THEN

                                'If the piece fits...
                                IF Validate(piece2, r, 0) = 1 THEN

                                    IF sfx = 1 THEN _SNDPLAY piecedown&

                                    _PUTIMAGE (0, 0), back&: _DISPLAY
                                    AddToScore piece2
                                    piece2 = 0 'remove piece from pen
                                ELSE
                                    curpiece = piece2
                                    GOSUB PieceReturn
                                    _PUTIMAGE (0, 0), undo&: _DISPLAY
                                END IF

                                GOTO doneamove

                            END IF
                        NEXT
                        curpiece = piece2
                        GOSUB PieceReturn

                        _PUTIMAGE (0, 0), undo&: _DISPLAY

                        GOTO doneamove

                    END IF

                LOOP
                'please release me, let me go....
                _FREEIMAGE undo&
                _FREEIMAGE back&
            END IF

        END IF

        'if clicked holding pen 3
        IF mx >= 610 AND mx <= 790 AND my >= 460 AND my <= 640 THEN

            IF piece3 <> 0 THEN

                'grab background in case restore needed
                undo& = _COPYIMAGE(_DISPLAY)

                'clear holding pen 3
                LINE (610, 460)-(790, 640), _RGB(105, 14, 0), BF: _DISPLAY

                back& = _COPYIMAGE(_DISPLAY)

                IF sfx = 1 THEN _SNDPLAY pieceup&

                'pickup piece, drag it around until mouse up
                DO
                    'show piece in hand, over backgraound
                    IF INT(RND * 3) = 0 THEN
                        _PUTIMAGE (0, 0), back&
                        '_PUTIMAGE (_MOUSEX, _MOUSEY), litesquare&: _DISPLAY
                        DrawPiece piece3, _MOUSEX, _MOUSEY
                    END IF

                    'If user unclicked the button (let go)
                    t2 = _MOUSEINPUT
                    IF _MOUSEBUTTON(1) = 0 THEN
                        mx2 = _MOUSEX: my2 = _MOUSEY

                        'see if unclicked over an empty box
                        FOR r = 1 TO rows * cols
                            IF mx2 >= box.x1(r) AND mx2 <= box.x2(r) AND my2 >= box.y1(r) AND my2 <= box.y2(r) THEN

                                'If the piece fits...
                                IF Validate(piece3, r, 0) = 1 THEN
                                    IF sfx = 1 THEN _SNDPLAY piecedown&
                                    _PUTIMAGE (0, 0), back&: _DISPLAY
                                    AddToScore piece3
                                    piece3 = 0 'remove piece from pen
                                ELSE
                                    curpiece = piece3
                                    GOSUB PieceReturn
                                    _PUTIMAGE (0, 0), undo&: _DISPLAY
                                END IF

                                GOTO doneamove

                            END IF
                        NEXT
                        curpiece = piece3
                        GOSUB PieceReturn

                        _PUTIMAGE (0, 0), undo&: _DISPLAY

                        GOTO doneamove

                    END IF

                LOOP
                'please release me, let me go....
                _FREEIMAGE undo&
                _FREEIMAGE back&
            END IF

        END IF

        doneamove:

        'Get new pieces if all holding pens empty
        IF piece1 = 0 AND piece2 = 0 AND piece3 = 0 THEN DrawHoldingPens

        UpdateBoard

        IF MovesLeft = 0 THEN
            IF score = hiscore THEN
                _PUTIMAGE (200, 150), outofmoves2&
            ELSE
                _PUTIMAGE (200, 150), outofmoves1&
            END IF
            PPRINT 380, 423, 24, _RGB(214, 156, 90), 0, LTRIM$(STR$(score))
            _AUTODISPLAY
            'wait until mouse click
            DO: k = _MOUSEINPUT
                IF INKEY$ = CHR$(32) THEN EXIT DO
            LOOP UNTIL _MOUSEBUTTON(1)
            score = 0
            'erase box values
            bc = 1 'counter
            FOR c = 1 TO cols
                FOR r = 1 TO rows
                    box(bc) = 0
                    bc = bc + 1
                NEXT
            NEXT
            SaveSettings
            firststart = 1: GOTO Restart
        END IF


    END IF

    keyin$ = UCASE$(INKEY$)
    IF keyin$ = CHR$(32) THEN
        score = 0
        'erase box values
        bc = 1 'counter
        FOR c = 1 TO cols
            FOR r = 1 TO rows
                box(bc) = 0
                bc = bc + 1
            NEXT
        NEXT
        SaveSettings
        GOTO Restart
    END IF
    IF keyin$ = CHR$(27) THEN EXIT DO
    IF keyin$ = "S" THEN
        IF sfx = 1 THEN
            sfx = 0: sfx$ = "OFF"
        ELSE
            sfx = 1: sfx$ = "ON"
        END IF
        back& = _COPYIMAGE(_DISPLAY)
        LINE (200, 175)-(530, 250), _RGBA(32, 0, 0, 150), BF
        LINE (200, 175)-(530, 250), _RGB(255, 255, 255), B
        LINE (199, 174)-(531, 251), _RGB(255, 255, 255), B
        PPRINT 240, 200, 20, _RGB(255, 255, 255), 0, "SFX TURNED " + sfx$
        _DISPLAY: _DELAY 2
        _PUTIMAGE (0, 0), back&
        _FREEIMAGE back&
    END IF

    IF _MOUSEBUTTON(2) OR keyin$ = "M" THEN
        MainMenu
        IF score = 0 THEN GOTO Restart
    END IF

    IF _MOUSEX > 657 AND _MOUSEX < 767 AND _MOUSEY > 5 AND _MOUSEY < 43 AND _MOUSEBUTTON(1) THEN
        MainMenu
        IF score = 0 THEN GOTO Restart
    END IF

    _AUTODISPLAY

LOOP

SYSTEM



'============================
'============
PieceReturn:

'animate sending piece back to original x/y
cx = mx2: cy = my2
xdone = 0: ydone = 0
DO
    IF mx2 > mx THEN
        cx = cx - 12
        IF cx < mx THEN
            cx = mx: xdone = 1
        END IF
    ELSE
        cx = cx + 12
        IF cx > mx THEN
            cx = mx: xdone = 1
        END IF
    END IF

    IF my2 > my THEN
        cy = cy - 12
        IF cy < my THEN
            cy = my: ydone = 1
        END IF
    ELSE
        cy = cy + 12
        IF cy > my THEN
            cy = my: ydone = 1
        END IF

    END IF

    _PUTIMAGE (0, 0), back&
    DrawPiece curpiece, cx - 20, cy - 20

    _DISPLAY

    _DELAY .0075

LOOP UNTIL xdone = 1 AND ydone = 1

RETURN


FUNCTION MovesLeft ()

    'Checks if moves are left

    MovesLeft = 0 'default none left until a move found

    'if there's a piece in holding pen 1
    IF piece1 <> 0 THEN

        'run piece check on all open boxes
        FOR r = 1 TO rows * cols
            'test piece on box if empty
            IF box(r) = 0 THEN
                IF Validate(piece1, r, 1) = 1 THEN
                    MovesLeft = 1: EXIT FOR
                END IF
            END IF
        NEXT
    END IF

    'if there's a piece in holding pen 2
    IF piece2 <> 0 THEN
        'run piece check on all open boxes
        FOR r = 1 TO rows * cols
            IF box(r) = 0 THEN
                IF Validate(piece2, r, 1) = 1 THEN
                    MovesLeft = 1: EXIT FOR
                END IF
            END IF
        NEXT
    END IF

    'if there's a piece in holding pen 3
    IF piece3 <> 0 THEN
        'run piece check on all open boxes
        FOR r = 1 TO rows * cols
            IF box(r) = 0 THEN
                IF Validate(piece3, r, 1) = 1 THEN
                    MovesLeft = 1: EXIT FOR
                END IF
            END IF
        NEXT
    END IF

END FUNCTION


SUB UpdateBoard

    _PUTIMAGE (box.x1(1) + 1, box.y1(1) - 1), backboard&

    bc = 1 'counter
    FOR c = 1 TO cols
        FOR r = 1 TO rows
            IF box(bc) <> 0 THEN
                _PUTIMAGE (box.x1(bc), box.y1(bc))-(box.x2(bc), box.y2(bc)), litesquare&
            END IF
            bc = bc + 1
        NEXT
    NEXT

    _DISPLAY

    'Check for completed lines...add score
    '10 points for clearing 1 row or col,
    'bonus points for doing several at one time
    tallyup = 0: bonus = 0
    'scan for completed rows...
    REDIM rowdo(10) AS INTEGER: rc = 1: rowhere = 0
    FOR rw = 1 TO 100 STEP 10
        IF RowDone(rw) THEN
            'RowClear rw
            rowdo(rc) = rw: rc = rc + 1: rowhere = 1
            tallyup = tallyup + 10
            bonus = bonus + 1
        END IF
    NEXT
    'scan for completed columns...
    REDIM coldo(10) AS INTEGER: cc = 1: colhere = 0
    FOR c = 1 TO 10
        IF ColDone(c) THEN
            coldo(cc) = c: cc = cc + 1: colhere = 1
            tallyup = tallyup + 10
            bonus = bonus + 1
            'ColClear c
        END IF
    NEXT
    'Now do the erasing...
    IF rowhere = 1 THEN
        FOR e = 1 TO 10
            IF rowdo(e) <> 0 THEN RowClear rowdo(e)
        NEXT
    END IF: ERASE rowdo
    IF colhere = 1 THEN
        FOR e = 1 TO 10
            IF coldo(e) <> 0 THEN ColClear coldo(e)
        NEXT
    END IF: ERASE coldo

    score = score + (tallyup * bonus)

    IF score > hiscore THEN
        hiscore = score
        SaveSettings
    END IF

    ShowScore

    _ICON _DISPLAY 'update app icon on taskbar

    'give messages for big scores
    IF tallyup >= 10 THEN
        back2& = _COPYIMAGE(_DISPLAY)

        'wait until mouse button up to continue
        DO: MI = _MOUSEINPUT
        LOOP UNTIL _MOUSEBUTTON(1) = 0

        IF tallyup = 30 THEN PPRINT 230, 230, 45, _RGB(255, 255, 255), 0, "Nice!"
        IF tallyup = 40 THEN PPRINT 230, 230, 45, _RGB(255, 255, 255), 0, "Great!"
        IF tallyup > 40 THEN PPRINT 200, 230, 45, _RGB(255, 255, 255), 0, "Awesome!"
        _DISPLAY

        DO: MI = _MOUSEINPUT
            IF _MOUSEBUTTON(1) OR _MOUSEBUTTON(2) THEN EXIT DO
        LOOP

        _PUTIMAGE (0, 0), back2&
        _FREEIMAGE back2&
        _DISPLAY

    END IF

END SUB


FUNCTION RowDone (row)
    RowDone = 1
    FOR r = row TO row + 9
        IF box(r) = 0 THEN RowDone = 0
    NEXT
END FUNCTION

SUB RowClear (row)
    FOR r = row TO row + 9
        LINE (box.x1(r), box.y1(r))-(box.x2(r), box.y2(r)), _RGB(255, 255, 255), BF
        _DISPLAY: _DELAY .02
        box(r) = 0

        IF sfx = 1 THEN
            IF INT(RND * 2) = 1 THEN
                _SNDPLAY pieceup&
            ELSE
                _SNDPLAY piecedown&
            END IF
        END IF

        _PUTIMAGE (box.x1(1) + 1, box.y1(1) - 1), backboard&
        bc = 1 'counter
        FOR c = 1 TO cols
            FOR r2 = 1 TO rows
                IF box(bc) <> 0 THEN
                    _PUTIMAGE (box.x1(bc), box.y1(bc))-(box.x2(bc), box.y2(bc)), litesquare&
                END IF
                bc = bc + 1
            NEXT
        NEXT

    NEXT
END SUB

FUNCTION ColDone (col)
    ColDone = 1
    FOR c = col TO (rows * cols) STEP 10
        IF box(c) = 0 THEN ColDone = 0
    NEXT
END FUNCTION

SUB ColClear (col)
    FOR r = col TO (rows * cols) STEP 10
        LINE (box.x1(r), box.y1(r))-(box.x2(r), box.y2(r)), _RGB(255, 255, 255), BF
        _DISPLAY: _DELAY .02
        box(r) = 0

        IF sfx = 1 THEN
            IF INT(RND * 2) = 1 THEN
                _SNDPLAY pieceup&
            ELSE
                _SNDPLAY piecedown&
            END IF
        END IF

        _PUTIMAGE (box.x1(1) + 1, box.y1(1) - 1), backboard&
        bc = 1 'counter
        FOR c = 1 TO cols
            FOR r2 = 1 TO rows
                IF box(bc) <> 0 THEN
                    _PUTIMAGE (box.x1(bc), box.y1(bc))-(box.x2(bc), box.y2(bc)), litesquare&
                END IF
                bc = bc + 1
            NEXT
        NEXT

    NEXT
END SUB

FUNCTION Validate (piece, num, checkonly)

    'This function validates if pieces fit
    'if checkonly flag is on, then it just checks to see
    'if checkonly flag = 0 then it checks and marks box() filled

    Validate = 0 'default not validated

    'one dot piece
    IF piece = 1 THEN
        IF box(num) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1
            END IF
            Validate = 1
        END IF
    END IF

    '2 dot horizonal
    IF piece = 2 THEN
        'don't validate if on the last box on right side
        SELECT CASE num
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        'validate if this box and next box over is free
        IF box(num) = 0 AND box(num + 1) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1
            END IF
            Validate = 1
        END IF
    END IF

    '2 dot vertical
    IF piece = 3 THEN
        'don't validate if on bottom row
        SELECT CASE num
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        'validate if this box and box below is free
        IF box(num) = 0 AND box(num + 10) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 10) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 4 THEN '3 dots left/up
        'dont validate if on top row
        SELECT CASE num
            CASE 1 TO 10: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 1) = 0 AND box(num - 9) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1: box(num - 9) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 5 THEN '3 dots right/up
        'don't validate bottom row
        SELECT CASE num
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        'don't validate if on the last box on right side
        SELECT CASE num
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 10) = 0 AND box(num + 11) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 10) = 1: box(num + 11) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 6 THEN
        'don't validate bottom row
        SELECT CASE num
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        'don't validate if on the last box on right side
        SELECT CASE num
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 1) = 0 AND box(num + 11) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1: box(num + 11) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 7 THEN
        'don't validate bottom row
        SELECT CASE num
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        'don't validate if on the last box on right side
        SELECT CASE num
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 1) = 0 AND box(num + 10) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1: box(num + 10) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 8 THEN '3 dots horz
        'don't validate if on the last 2 columns on right side
        SELECT CASE num
            CASE 9, 19, 29, 39, 49, 59, 69, 79, 89, 99: Validate = 0: EXIT FUNCTION
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 1) = 0 AND box(num + 2) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1: box(num + 2) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 9 THEN '3 dots vert
        'don't validate bottom 2 row
        SELECT CASE num
            CASE 81 TO 90: Validate = 0: EXIT FUNCTION
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 10) = 0 AND box(num + 20) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 10) = 1: box(num + 20) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 10 THEN '4 dots block
        'don't validate bottom row
        SELECT CASE num
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        'don't validate if on the last box on right side
        SELECT CASE num
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 1) = 0 AND box(num + 10) = 0 AND box(num + 11) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1: box(num + 10) = 1: box(num + 11) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 11 THEN '4 dots horz
        'don't validate if on the last 3 columns on right side
        SELECT CASE num
            CASE 8, 18, 28, 38, 48, 58, 68, 78, 88, 98: Validate = 0: EXIT FUNCTION
            CASE 9, 19, 29, 39, 49, 59, 69, 79, 89, 99: Validate = 0: EXIT FUNCTION
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 1) = 0 AND box(num + 2) = 0 AND box(num + 3) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1: box(num + 2) = 1: box(num + 3) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 12 THEN '4 dots vert
        'don't validate bottom 3 rows
        SELECT CASE num
            CASE 71 TO 80: Validate = 0: EXIT FUNCTION
            CASE 81 TO 90: Validate = 0: EXIT FUNCTION
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 10) = 0 AND box(num + 20) = 0 AND box(num + 30) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 10) = 1: box(num + 20) = 1: box(num + 30) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 13 THEN '5 dots horz
        'don't validate if on the last 3 columns on right side
        SELECT CASE num
            CASE 7, 17, 27, 37, 47, 57, 67, 77, 87, 97: Validate = 0: EXIT FUNCTION
            CASE 8, 18, 28, 38, 48, 58, 68, 78, 88, 98: Validate = 0: EXIT FUNCTION
            CASE 9, 19, 29, 39, 49, 59, 69, 79, 89, 99: Validate = 0: EXIT FUNCTION
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 1) = 0 AND box(num + 2) = 0 AND box(num + 3) = 0 AND box(num + 4) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1: box(num + 2) = 1: box(num + 3) = 1: box(num + 4) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 14 THEN '5 dots vert
        'don't validate bottom 4 rows
        SELECT CASE num
            CASE 61 TO 70: Validate = 0: EXIT FUNCTION
            CASE 71 TO 80: Validate = 0: EXIT FUNCTION
            CASE 81 TO 90: Validate = 0: EXIT FUNCTION
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 10) = 0 AND box(num + 20) = 0 AND box(num + 30) = 0 AND box(num + 40) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 10) = 1: box(num + 20) = 1: box(num + 30) = 1: box(num + 40) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 15 THEN '5 dots left/up
        'don't validate bottom 2 rows
        SELECT CASE num
            CASE 81 TO 90: Validate = 0: EXIT FUNCTION
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        'don't validate if on the last 2 columns on right side
        SELECT CASE num
            CASE 9, 19, 29, 39, 49, 59, 69, 79, 89, 99: Validate = 0: EXIT FUNCTION
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 1) = 0 AND box(num + 2) = 0 AND box(num + 12) = 0 AND box(num + 22) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1: box(num + 2) = 1: box(num + 12) = 1: box(num + 22) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 16 THEN '5 dots right/up
        'don't validate bottom 2 rows
        SELECT CASE num
            CASE 81 TO 90: Validate = 0: EXIT FUNCTION
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        'don't validate if on the last 2 columns on right side
        SELECT CASE num
            CASE 9, 19, 29, 39, 49, 59, 69, 79, 89, 99: Validate = 0: EXIT FUNCTION
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 1) = 0 AND box(num + 2) = 0 AND box(num + 10) = 0 AND box(num + 20) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1: box(num + 2) = 1: box(num + 10) = 1: box(num + 20) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 17 THEN '5 dots right/down
        'don't validate bottom 2 rows
        SELECT CASE num
            CASE 81 TO 90: Validate = 0: EXIT FUNCTION
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        'don't validate if on the last 2 columns on right side
        SELECT CASE num
            CASE 9, 19, 29, 39, 49, 59, 69, 79, 89, 99: Validate = 0: EXIT FUNCTION
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 10) = 0 AND box(num + 20) = 0 AND box(num + 21) = 0 AND box(num + 22) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 10) = 1: box(num + 20) = 1: box(num + 21) = 1: box(num + 22) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 18 THEN '5 dots left/down
        'don't validate top 2 rows
        SELECT CASE num
            CASE 1 TO 10: Validate = 0: EXIT FUNCTION
            CASE 11 TO 20: Validate = 0: EXIT FUNCTION
        END SELECT
        'don't validate if on the last 2 columns on right side
        SELECT CASE num
            CASE 9, 19, 29, 39, 49, 59, 69, 79, 89, 99: Validate = 0: EXIT FUNCTION
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 1) = 0 AND box(num + 2) = 0 AND box(num - 8) = 0 AND box(num - 18) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1: box(num + 2) = 1: box(num - 8) = 1: box(num - 18) = 1
            END IF
            Validate = 1
        END IF
    END IF

    IF piece = 19 THEN '9 dots big block
        'don't validate bottom 2 rows
        SELECT CASE num
            CASE 81 TO 90: Validate = 0: EXIT FUNCTION
            CASE 91 TO 100: Validate = 0: EXIT FUNCTION
        END SELECT
        'don't validate if on the last 2 columns on right side
        SELECT CASE num
            CASE 9, 19, 29, 39, 49, 59, 69, 79, 89, 99: Validate = 0: EXIT FUNCTION
            CASE 10, 20, 30, 40, 50, 60, 70, 80, 90, 100: Validate = 0: EXIT FUNCTION
        END SELECT
        IF box(num) = 0 AND box(num + 1) = 0 AND box(num + 2) = 0 AND box(num + 10) = 0 AND box(num + 11) = 0 AND box(num + 12) = 0 AND box(num + 20) = 0 AND box(num + 21) = 0 AND box(num + 22) = 0 THEN
            IF checkonly = 0 THEN
                box(num) = 1: box(num + 1) = 1: box(num + 2) = 1: box(num + 10) = 1: box(num + 11) = 1: box(num + 12) = 1: box(num + 20) = 1: box(num + 21) = 1: box(num + 22) = 1
            END IF
            Validate = 1
        END IF
    END IF

END FUNCTION

SUB AddToScore (num)

    IF num = 1 THEN score = score + 1
    IF num = 2 THEN score = score + 2
    IF num = 3 THEN score = score + 2
    IF num = 4 THEN score = score + 3
    IF num = 5 THEN score = score + 3
    IF num = 6 THEN score = score + 3
    IF num = 7 THEN score = score + 3
    IF num = 8 THEN score = score + 3
    IF num = 9 THEN score = score + 3
    IF num = 10 THEN score = score + 4
    IF num = 11 THEN score = score + 4
    IF num = 12 THEN score = score + 4
    IF num = 13 THEN score = score + 5
    IF num = 14 THEN score = score + 5
    IF num = 15 THEN score = score + 5
    IF num = 16 THEN score = score + 5
    IF num = 17 THEN score = score + 5
    IF num = 18 THEN score = score + 5
    IF num = 19 THEN score = score + 9

END SUB

SUB DrawHoldingPens

    'Draws 3 holding pens, smaller version of pieces,
    'and assigns random piece to each holding pen...

    'holding pen 1
    LINE (610, 60)-(790, 240), _RGB(105, 14, 0), BF
    'holding pen 2
    LINE (610, 260)-(790, 440), _RGB(105, 14, 0), BF
    'holding pen 3
    LINE (610, 460)-(790, 640), _RGB(105, 14, 0), BF

    FOR pens = 1 TO 3

        'Get a random piece
        'There are 19 total pieces to choose from
        piece = INT(RND * 19) + 1

        'y offset vaule for the pens
        IF pens = 1 THEN yo = 50
        IF pens = 2 THEN yo = 250
        IF pens = 3 THEN yo = 450

        IF piece = 1 THEN '1 dot
            _PUTIMAGE (685, 85 + yo)-(715, 115 + yo), litesquare&
        END IF
        IF piece = 2 THEN '2 dots horizonal
            _PUTIMAGE (670, 85 + yo)-(700, 115 + yo), litesquare& 'mid
            _PUTIMAGE (700, 85 + yo)-(730, 115 + yo), litesquare& 'right
        END IF
        IF piece = 3 THEN '2 dots vertical
            _PUTIMAGE (685, 100 + yo)-(715, 130 + yo), litesquare& 'mid
            _PUTIMAGE (685, 70 + yo)-(715, 100 + yo), litesquare& 'up
        END IF
        IF piece = 4 THEN '3 dots Left/Up
            _PUTIMAGE (700, 100 + yo)-(730, 130 + yo), litesquare& 'mid
            _PUTIMAGE (700, 70 + yo)-(730, 100 + yo), litesquare& 'up
            _PUTIMAGE (670, 100 + yo)-(700, 130 + yo), litesquare& 'left
        END IF
        IF piece = 5 THEN '3 dots right/Up
            _PUTIMAGE (670, 100 + yo)-(700, 130 + yo), litesquare& 'mid
            _PUTIMAGE (670, 70 + yo)-(700, 100 + yo), litesquare& 'up
            _PUTIMAGE (700, 100 + yo)-(730, 130 + yo), litesquare& 'right
        END IF
        IF piece = 6 THEN '3 dots Left/down
            _PUTIMAGE (700, 70 + yo)-(730, 100 + yo), litesquare& 'mid
            _PUTIMAGE (700, 100 + yo)-(730, 130 + yo), litesquare& 'down
            _PUTIMAGE (670, 70 + yo)-(700, 100 + yo), litesquare& 'left
        END IF
        IF piece = 7 THEN '3 dots right/down
            _PUTIMAGE (670, 70 + yo)-(700, 100 + yo), litesquare& 'mid
            _PUTIMAGE (670, 100 + yo)-(700, 130 + yo), litesquare& 'down
            _PUTIMAGE (700, 70 + yo)-(730, 100 + yo), litesquare& 'left
        END IF
        IF piece = 8 THEN '3 dots horizonal
            _PUTIMAGE (685, 85 + yo)-(715, 115 + yo), litesquare& 'mid
            _PUTIMAGE (715, 85 + yo)-(745, 115 + yo), litesquare& 'right
            _PUTIMAGE (655, 85 + yo)-(685, 115 + yo), litesquare& 'left
        END IF
        IF piece = 9 THEN '3 dots vertical
            _PUTIMAGE (685, 85 + yo)-(715, 115 + yo), litesquare& 'mid
            _PUTIMAGE (685, 55 + yo)-(715, 85 + yo), litesquare& 'up
            _PUTIMAGE (685, 115 + yo)-(715, 145 + yo), litesquare& 'down
        END IF
        IF piece = 10 THEN '4 dots block
            _PUTIMAGE (665, 75 + yo)-(695, 105 + yo), litesquare& 'top left
            _PUTIMAGE (695, 75 + yo)-(725, 105 + yo), litesquare& 'top right
            _PUTIMAGE (665, 105 + yo)-(695, 135 + yo), litesquare& 'bottom left
            _PUTIMAGE (695, 105 + yo)-(725, 135 + yo), litesquare& 'bottom right
        END IF
        IF piece = 11 THEN '4 dots horizonal
            _PUTIMAGE (640, 85 + yo)-(670, 115 + yo), litesquare&
            _PUTIMAGE (670, 85 + yo)-(700, 115 + yo), litesquare&
            _PUTIMAGE (700, 85 + yo)-(730, 115 + yo), litesquare&
            _PUTIMAGE (730, 85 + yo)-(760, 115 + yo), litesquare&
        END IF
        IF piece = 12 THEN '4 dots vertical
            _PUTIMAGE (685, 40 + yo)-(715, 70 + yo), litesquare&
            _PUTIMAGE (685, 70 + yo)-(715, 100 + yo), litesquare&
            _PUTIMAGE (685, 100 + yo)-(715, 130 + yo), litesquare&
            _PUTIMAGE (685, 130 + yo)-(715, 160 + yo), litesquare&
        END IF
        IF piece = 13 THEN '5 dots horizonal
            _PUTIMAGE (625, 85 + yo)-(655, 115 + yo), litesquare&
            _PUTIMAGE (655, 85 + yo)-(685, 115 + yo), litesquare&
            _PUTIMAGE (685, 85 + yo)-(715, 115 + yo), litesquare&
            _PUTIMAGE (715, 85 + yo)-(745, 115 + yo), litesquare&
            _PUTIMAGE (745, 85 + yo)-(775, 115 + yo), litesquare&
        END IF
        IF piece = 14 THEN '5 dots vertical
            _PUTIMAGE (685, 20 + yo)-(715, 50 + yo), litesquare&
            _PUTIMAGE (685, 50 + yo)-(715, 85 + yo), litesquare&
            _PUTIMAGE (685, 85 + yo)-(715, 115 + yo), litesquare&
            _PUTIMAGE (685, 115 + yo)-(715, 145 + yo), litesquare&
            _PUTIMAGE (685, 145 + yo)-(715, 175 + yo), litesquare&
        END IF
        IF piece = 15 THEN '5 dots left/up
            _PUTIMAGE (655, 55 + yo)-(685, 85 + yo), litesquare&
            _PUTIMAGE (685, 55 + yo)-(715, 85 + yo), litesquare&
            _PUTIMAGE (715, 55 + yo)-(745, 85 + yo), litesquare&
            _PUTIMAGE (715, 85 + yo)-(745, 115 + yo), litesquare&
            _PUTIMAGE (715, 115 + yo)-(745, 145 + yo), litesquare&
        END IF
        IF piece = 16 THEN '5 dots right/up
            _PUTIMAGE (655, 55 + yo)-(685, 85 + yo), litesquare&
            _PUTIMAGE (685, 55 + yo)-(715, 85 + yo), litesquare&
            _PUTIMAGE (715, 55 + yo)-(745, 85 + yo), litesquare&
            _PUTIMAGE (655, 85 + yo)-(685, 115 + yo), litesquare&
            _PUTIMAGE (655, 115 + yo)-(685, 145 + yo), litesquare&
        END IF
        IF piece = 17 THEN '5 dots right/down
            _PUTIMAGE (655, 55 + yo)-(685, 85 + yo), litesquare&
            _PUTIMAGE (685, 115 + yo)-(715, 145 + yo), litesquare&
            _PUTIMAGE (715, 115 + yo)-(745, 145 + yo), litesquare&
            _PUTIMAGE (655, 85 + yo)-(685, 115 + yo), litesquare&
            _PUTIMAGE (655, 115 + yo)-(685, 145 + yo), litesquare&
        END IF
        IF piece = 18 THEN '5 dots left/down
            _PUTIMAGE (715, 55 + yo)-(745, 85 + yo), litesquare&
            _PUTIMAGE (715, 85 + yo)-(745, 115 + yo), litesquare&
            _PUTIMAGE (685, 115 + yo)-(715, 145 + yo), litesquare&
            _PUTIMAGE (715, 115 + yo)-(745, 145 + yo), litesquare&
            _PUTIMAGE (655, 115 + yo)-(685, 145 + yo), litesquare&
        END IF
        IF piece = 19 THEN '9 dots big block
            _PUTIMAGE (655, 55 + yo)-(685, 85 + yo), litesquare&
            _PUTIMAGE (685, 55 + yo)-(715, 85 + yo), litesquare&
            _PUTIMAGE (655, 85 + yo)-(685, 115 + yo), litesquare&
            _PUTIMAGE (685, 85 + yo)-(715, 115 + yo), litesquare&
            _PUTIMAGE (715, 55 + yo)-(745, 85 + yo), litesquare&
            _PUTIMAGE (715, 85 + yo)-(745, 115 + yo), litesquare&
            _PUTIMAGE (685, 115 + yo)-(715, 145 + yo), litesquare&
            _PUTIMAGE (715, 115 + yo)-(745, 145 + yo), litesquare&
            _PUTIMAGE (655, 115 + yo)-(685, 145 + yo), litesquare&
        END IF

        'Assign the generated pieces values
        IF pens = 1 THEN piece1 = piece
        IF pens = 2 THEN piece2 = piece
        IF pens = 3 THEN piece3 = piece

    NEXT

    _DISPLAY

END SUB


SUB DrawPiece (piece, x, y)

    x = x - 30: y = y - 30 'center the mouse on a block


    IF piece = 1 THEN '1 dot
        _PUTIMAGE (x, y), litesquare&
    END IF

    IF piece = 2 THEN '2 dots horizonal
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
    END IF

    IF piece = 3 THEN '2 dot vertical
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x, y + 60), litesquare&
    END IF

    IF piece = 4 THEN '3 dots left/up
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
        _PUTIMAGE (x + 60, y - 60), litesquare&
    END IF

    IF piece = 5 THEN '3 dots right/up
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x, y + 60), litesquare&
        _PUTIMAGE (x + 60, y + 60), litesquare&
    END IF

    IF piece = 6 THEN '3 dots left/down
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
        _PUTIMAGE (x + 60, y + 60), litesquare&
    END IF

    IF piece = 7 THEN '3 dots right/down
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
        _PUTIMAGE (x, y + 60), litesquare&
    END IF

    IF piece = 8 THEN '3 dots horizonal
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60, y), litesquare&
    END IF

    IF piece = 9 THEN '3 dots vert
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x, y + 60), litesquare&
        _PUTIMAGE (x, y + 60 + 60), litesquare&
    END IF

    IF piece = 10 THEN '4 dots block
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
        _PUTIMAGE (x, y + 60), litesquare&
        _PUTIMAGE (x + 60, y + 60), litesquare&
    END IF

    IF piece = 11 THEN '4 dots horz
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60 + 60, y), litesquare&
    END IF

    IF piece = 12 THEN '4 dots vert
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x, y + 60), litesquare&
        _PUTIMAGE (x, y + 60 + 60), litesquare&
        _PUTIMAGE (x, y + 60 + 60 + 60), litesquare&
    END IF

    IF piece = 13 THEN '5 dots horz
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60 + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60 + 60 + 60, y), litesquare&
    END IF

    IF piece = 14 THEN '5 dots vert
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x, y + 60), litesquare&
        _PUTIMAGE (x, y + 60 + 60), litesquare&
        _PUTIMAGE (x, y + 60 + 60 + 60), litesquare&
        _PUTIMAGE (x, y + 60 + 60 + 60 + 60), litesquare&
    END IF

    IF piece = 15 THEN '5 dots left/up
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60, y + 60), litesquare&
        _PUTIMAGE (x + 60 + 60, y + 60 + 60), litesquare&
    END IF

    IF piece = 16 THEN '5 dots right/up
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60, y), litesquare&
        _PUTIMAGE (x, y + 60), litesquare&
        _PUTIMAGE (x, y + 60 + 60), litesquare&
    END IF

    IF piece = 17 THEN '5 dots right/down
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x, y + 60), litesquare&
        _PUTIMAGE (x, y + 60 + 60), litesquare&
        _PUTIMAGE (x + 60, y + 60 + 60), litesquare&
        _PUTIMAGE (x + 60 + 60, y + 60 + 60), litesquare&
    END IF

    IF piece = 18 THEN '5 dots left/down
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60, y - 60), litesquare&
        _PUTIMAGE (x + 60 + 60, y - 60 - 60), litesquare&
    END IF

    IF piece = 19 THEN '9 dots big block
        _PUTIMAGE (x, y), litesquare&
        _PUTIMAGE (x + 60, y), litesquare&
        _PUTIMAGE (x + 60 + 60, y), litesquare&
        _PUTIMAGE (x, y + 60), litesquare&
        _PUTIMAGE (x + 60, y + 60), litesquare&
        _PUTIMAGE (x + 60 + 60, y + 60), litesquare&
        _PUTIMAGE (x, y + 60 + 60), litesquare&
        _PUTIMAGE (x + 60, y + 60 + 60), litesquare&
        _PUTIMAGE (x + 60 + 60, y + 60 + 60), litesquare&
    END IF

    _DISPLAY

END SUB

'======================================================================

SUB ShowScore
    LINE (2, 2)-(_WIDTH - 2, 48), _RGB(64, 0, 0), BF

    PPRINT 350, 11, 24, _RGB(214, 156, 90), _RGB(64, 0, 0), STR$(hiscore)
    PPRINT 110, 11, 24, _RGB(214, 156, 90), _RGB(64, 0, 0), STR$(score)
    _PUTIMAGE (670, 7), topmenu&
    _PUTIMAGE (310, 10), tophi&
    _PUTIMAGE (20, 8), topscore&
    _DISPLAY
    SaveSettings
END SUB

SUB PPRINT (x, y, size, clr&, trans&, text$)
    orig& = _DEST
    bit = 32: IF _PIXELSIZE(0) = 1 THEN bit = 256
    FOR t = 0 TO LEN(text$) - 1
        pprintimg& = _NEWIMAGE(16, 16, bit)
        _DEST pprintimg&
        CLS , trans&: COLOR clr&
        PRINT MID$(text$, t + 1, 1);
        _CLEARCOLOR _RGB(0, 0, 0), pprintimg&
        _DEST orig&
        x1 = x + (t * size): x2 = x1 + size
        y1 = y: y2 = y + size
        _PUTIMAGE (x1 - (size / 2), y1)-(x2, y2 + (size / 3)), pprintimg&
        _FREEIMAGE pprintimg&
    NEXT
END SUB

SUB SaveSettings
    IF _FILEEXISTS("10x10.dat") THEN KILL "10x10.dat"
    scr = FREEFILE
    OPEN "10x10.dat" FOR OUTPUT AS #scr
    hi$ = MKL$(hiscore)
    sc$ = MKL$(score)
    sf$ = MKL$(sfx)
    PRINT #scr, hi$;
    PRINT #scr, sc$;
    PRINT #scr, sf$;
    'save box values
    bc = 1 'counter
    FOR c = 1 TO cols
        FOR r = 1 TO rows
            b$ = LTRIM$(RTRIM$(STR$(box(bc))))
            PRINT #scr, b$;
            bc = bc + 1
        NEXT
    NEXT
    CLOSE #scr
END SUB

SUB MainMenu
    back2& = _COPYIMAGE(_DISPLAY)

    IF sfx = 1 THEN _SNDPLAY piecedown&

    _PUTIMAGE (200, 150), menu&
    _DISPLAY

    'clear mouse clicks
    DO: mi = _MOUSEINPUT
        IF _MOUSEBUTTON(1) = 0 AND _MOUSEBUTTON(2) = 0 THEN EXIT DO
    LOOP

    'get menu selection
    DO
        _PUTIMAGE (200, 150), menu&

        mi = _MOUSEINPUT
        mmx = _MOUSEX: mmy = _MOUSEY

        'hovering effect...
        'how to play
        IF mmx > 230 AND mmx < 515 AND mmy > 248 AND mmy < 295 THEN
            _PUTIMAGE (230, 248), menuitem1on&
        ELSE
            _PUTIMAGE (230, 248), menuitem1off&
        END IF
        'sfx on/off
        IF mmx > 230 AND mmx < 515 AND mmy > 300 AND mmy < 345 THEN
            _PUTIMAGE (230, 300), menuitem2on&
        ELSE
            _PUTIMAGE (230, 300), menuitem2off&
        END IF
        'reset hi
        IF mmx > 230 AND mmx < 515 AND mmy > 350 AND mmy < 390 THEN
            _PUTIMAGE (230, 350), menuitem3on&
        ELSE
            _PUTIMAGE (230, 350), menuitem3off&
        END IF
        'start new
        IF mmx > 230 AND mmx < 515 AND mmy > 400 AND mmy < 445 THEN
            _PUTIMAGE (230, 400), menuitem4on&
        ELSE
            _PUTIMAGE (230, 400), menuitem4off&
        END IF

        'show resume if game in progress
        IF score > 0 THEN
            IF mmx > 230 AND mmx < 515 AND mmy > 450 AND mmy < 495 THEN
                _PUTIMAGE (230, 450), menuitem5on&
            ELSE
                _PUTIMAGE (230, 450), menuitem5off&
            END IF
        END IF

        'Show ON of OFF based on sfx setting
        IF sfx = 1 THEN
            PPRINT 435, 310, 24, _RGB(100, 255, 100), 0, "ON"
        ELSE
            PPRINT 435, 310, 24, _RGB(255, 255, 100), 0, "OFF"
        END IF

        _DISPLAY

        'clicking on the menu
        IF _MOUSEBUTTON(1) THEN

            'how to play
            IF mmx > 230 AND mmx < 515 AND mmy > 248 AND mmy < 295 THEN
                IF sfx = 1 THEN _SNDPLAY pieceup&
                _PUTIMAGE (230, 248), menuitem1off&: _DISPLAY: _DELAY .2
                'wait until mouse button back up
                DO: k = _MOUSEINPUT: LOOP UNTIL _MOUSEBUTTON(1) = 0
                IF sfx = 1 THEN _SNDPLAY piecedown&
                _PUTIMAGE (230, 248), menuitem1on&: _DISPLAY: _DELAY .2

                _PUTIMAGE (200, 150), howto&: _DISPLAY

                'wait until mouse button clicked
                DO: k = _MOUSEINPUT: LOOP UNTIL _MOUSEBUTTON(1)

                'wait until mouse button back up
                DO: k = _MOUSEINPUT: LOOP UNTIL _MOUSEBUTTON(1) = 0

            END IF

            'if sfx change
            IF mmx > 230 AND mmx < 515 AND mmy > 300 AND mmy < 345 THEN
                IF sfx = 1 THEN _SNDPLAY pieceup&
                _PUTIMAGE (230, 300), menuitem2off&: _DISPLAY: _DELAY .2
                IF sfx = 1 THEN _SNDPLAY piecedown&
                _PUTIMAGE (230, 300), menuitem2on&: _DISPLAY: _DELAY .2
                IF sfx = 1 THEN
                    sfx = 0: sfx$ = "OFF"
                    PPRINT 435, 310, 24, _RGB(255, 255, 100), 0, "OFF"
                ELSE
                    sfx = 1: sfx$ = "ON"
                    IF sfx = 1 THEN _SNDPLAY pieceup&
                    PPRINT 435, 310, 24, _RGB(100, 255, 100), 0, "ON"
                END IF
                SaveSettings
                _DISPLAY

                'wait until mouse button back up
                DO: k = _MOUSEINPUT: LOOP UNTIL _MOUSEBUTTON(1) = 0

            END IF
            'if reset HI
            IF mmx > 230 AND mmx < 515 AND mmy > 350 AND mmy < 390 THEN
                IF sfx = 1 THEN _SNDPLAY pieceup&
                _PUTIMAGE (230, 350), menuitem3off&: _DISPLAY: _DELAY .2
                IF sfx = 1 THEN _SNDPLAY piecedown&
                _PUTIMAGE (230, 350), menuitem3on&: _DISPLAY: _DELAY .2
                hiscore = 0: SaveSettings
                LINE (200, 175)-(530, 250), _RGBA(32, 0, 0, 150), BF
                LINE (200, 175)-(530, 250), _RGB(255, 255, 255), B
                LINE (199, 174)-(531, 251), _RGB(255, 255, 255), B
                PPRINT 240, 200, 20, _RGB(255, 255, 255), 0, "HI SCORE RESET"
                SaveSettings
                ShowScore
                _DISPLAY: _DELAY 1.5
                'clear mouse clicks
                DO: mi = _MOUSEINPUT
                    IF _MOUSEBUTTON(1) = 0 AND _MOUSEBUTTON(2) = 0 THEN EXIT DO
                LOOP
            END IF

            'if start new
            IF mmx > 230 AND mmx < 515 AND mmy > 400 AND mmy < 445 THEN
                IF sfx = 1 THEN _SNDPLAY pieceup&
                _PUTIMAGE (230, 400), menuitem4off&: _DISPLAY: _DELAY .2
                IF sfx = 1 THEN _SNDPLAY piecedown&
                _PUTIMAGE (230, 400), menuitem4on&: _DISPLAY: _DELAY .2
                score = 0
                'erase box values
                bc = 1 'counter
                FOR c = 1 TO cols
                    FOR r = 1 TO rows
                        box(bc) = 0
                        bc = bc + 1
                    NEXT
                NEXT
                SaveSettings
                _PUTIMAGE (0, 0), back2&
                _DISPLAY
                _FREEIMAGE back2&
                'clear mouse clicks
                DO: mi = _MOUSEINPUT
                    IF _MOUSEBUTTON(1) = 0 AND _MOUSEBUTTON(2) = 0 THEN EXIT DO
                LOOP
                EXIT SUB
            END IF

            'if clicked resume
            IF score > 0 THEN
                IF mmx > 230 AND mmx < 515 AND mmy > 450 AND mmy < 495 THEN
                    IF sfx = 1 THEN _SNDPLAY pieceup&
                    _PUTIMAGE (230, 450), menuitem5off&: _DISPLAY: _DELAY .2
                    IF sfx = 1 THEN _SNDPLAY piecedown&
                    _PUTIMAGE (230, 450), menuitem5on&: _DISPLAY: _DELAY .2
                    _DISPLAY: _DELAY .2
                    EXIT DO
                END IF
            END IF

        END IF

    LOOP

    _PUTIMAGE (0, 0), back2&
    ShowScore
    _DISPLAY
    _FREEIMAGE back2&

    'clear mouse clicks
    DO: mi = _MOUSEINPUT
        IF _MOUSEBUTTON(1) = 0 AND _MOUSEBUTTON(2) = 0 THEN EXIT DO
    LOOP

    _KEYCLEAR 'just in case

END SUB

